@@ -22,6 +22,9 @@
 
 #include <QSet>
 #include <QDebug>
+#include <QCoreApplication>
+
+#include <libintl.h>
 
 // needed for gio includes.
 #undef signals
@@ -198,6 +201,33 @@ bool operator== (const ActionData &a, const ActionData &b) {
            a.desc    == b.desc;
 }
 
+namespace {
+class QuitAction: public Action {
+    Q_OBJECT
+};
+
+class GlobalActionContext: public ActionContext {
+public:
+    GlobalActionContext(QObject *parent = 0) :
+            ActionContext(parent) {
+    }
+
+    void addBuiltInAction(Action *action) {
+        built_in_actions << action;
+    }
+
+    QSet<Action *> allActions() {
+        return actions() + built_in_actions;
+    }
+private:
+    QSet<Action *> built_in_actions;
+};
+
+static inline char * _(const char *__msgid) {
+        return gettext(__msgid);
+}
+}
+
 //! \private
 class Q_DECL_HIDDEN unity::action::ActionManager::Private : public QObject
 {
@@ -208,9 +238,11 @@ public:
 
     QSet<Action *> actions;
 
-    ActionContext *globalContext;
+    GlobalActionContext *globalContext;
     QSet<ActionContext *> localContexts;
 
+    QScopedPointer<Action> quitAction;
+
     ActionContext *activeLocalContext;
 
     QHash<ActionContext *, ContextData> contextData;
@@ -225,7 +257,7 @@ public:
     Private(ActionManager *mgr)
         : q(mgr)
     {
-        globalContext = new ActionContext();
+        globalContext = new GlobalActionContext();
         hudManager = 0;
     }
     ~Private() {
@@ -337,7 +369,14 @@ ActionManager::ActionManager(QObject *parent)
 
     d->actionGroup = g_simple_action_group_new();
 
+    d->quitAction.reset(new QuitAction());
+    d->quitAction->setText(_("Quit"));
+    d->quitAction->setDescription(_("Quit the application"));
+    d->quitAction->setKeywords(_("Exit;Close"));
+    connect(d->quitAction.data(), SIGNAL(triggered(QVariant)), this, SIGNAL(quit()));
+
     d->createContext(d->globalContext);
+    d->globalContext->addBuiltInAction(d->quitAction.data());
     d->updateContext(d->globalContext);
     hud_manager_switch_window_context(d->hudManager,
                                       d->contextData[d->globalContext].publisher);
@@ -631,7 +670,7 @@ ActionManager::Private::updateActionGroup()
     // current actions in global context and
     // in the active local context
 
-    QSet<Action *> globalActions = globalContext->actions();
+    QSet<Action *> globalActions = globalContext->allActions();
     QSet<Action *> localActions;
     if (activeLocalContext)
         localActions = activeLocalContext->actions();
@@ -704,7 +743,12 @@ ActionManager::Private::updateContext(ActionContext *context)
     ContextData &cdata = contextData[context];
 
     QSet<Action *> oldActions = cdata.actions;
-    QSet<Action *> currentActions = context->actions();
+    QSet<Action *> currentActions;
+    if(context == globalContext) {
+        currentActions = globalContext->allActions();
+    } else {
+        currentActions = context->actions();
+    }
     QSet<Action *> removedActions = oldActions - currentActions;
 
     foreach (Action *action, currentActions) {
@@ -774,7 +818,7 @@ ActionManager::Private::updateHudContext(ActionContext *context, QSet<Action *>
     const ContextData &cdata = contextData[context];
 
 
-    QSet<Action *> currentActions = context->actions() + globalContext->actions();
+    QSet<Action *> currentActions = context->actions() + globalContext->allActions();
     QSet<Action *> newActions     = currentActions - oldActions;
     QSet<Action *> removedActions = oldActions - currentActions;
 
@@ -821,8 +865,9 @@ ActionManager::Private::contextDestroyed(QObject *obj)
                     << "\tin the client code. ";
         globalContext->disconnect(this);
         destroyContext(globalContext);
-        globalContext = new ActionContext();
+        globalContext = new GlobalActionContext();
         createContext(globalContext);
+        globalContext->addBuiltInAction(quitAction.data());
         updateContext(globalContext);
         connect(globalContext, SIGNAL(actionsChanged()), this, SLOT(contextActionsChanged()));
         connect(globalContext, SIGNAL(activeChanged(bool)), this, SLOT(contextActiveChanged(bool)));
@@ -1022,6 +1067,13 @@ ActionManager::Private::updateActionDescription(Action *action,
                                                    "commitLabel",
                                                    g_variant_new_string(qPrintable(previewAction->commitLabel())));
     }
+
+    QuitAction *quitAction = qobject_cast<QuitAction *>(action);
+    if (quitAction != 0) {
+        hud_action_description_set_attribute_value(desc,
+                                                   "hud-toolbar-item",
+                                                   g_variant_new_string("quit"));
+    }
 }
 
 void
@@ -1038,7 +1090,7 @@ ActionManager::Private::updateActionsWhenNameOrTypeHaveChanged(Action *action)
     g_signal_handlers_disconnect_by_data(G_OBJECT(adata.gaction), action);
     g_clear_object(&adata.gaction);
     adata.gaction = (GSimpleAction *)g_object_ref(tmpdata.gaction);
-    if (globalContext->actions().contains(action) ||
+    if (globalContext->allActions().contains(action) ||
         (activeLocalContext != 0 && activeLocalContext->actions().contains(action))) {
         updateActionGroup();
     }
